@@ -3,22 +3,22 @@
    Copyright (C) 2000 Thomas Moestl
    Copyright (C) 2003, 2004, 2005 Paul A. Rombouts
 
-This file is part of the pdnsd package.
+  This file is part of the pdnsd package.
 
-pdnsd is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+  pdnsd is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 3 of the License, or
+  (at your option) any later version.
 
-pdnsd is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+  pdnsd is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with pdsnd; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+  You should have received a copy of the GNU General Public License
+  along with pdnsd; see the file COPYING. If not, see
+  <http://www.gnu.org/licenses/>.
+*/
 
 /* $Id: cache.h,v 1.11 2001/05/09 17:51:52 tmm Exp $ */
 
@@ -39,7 +39,12 @@ struct rr_lent_s;
  */
 typedef struct rr_b_s  {
 	struct rr_b_s    *next;                   /* this is the next pointer in the dns_cent_t list. */
-	unsigned short   rdlen;
+	unsigned         rdlen;
+#if ALLOW_LOCAL_AAAA || defined(ENABLE_IPV6)
+	struct in6_addr  data[0];                 /* dummy for alignment */
+#else
+	struct in_addr   data[0];
+#endif
 } rr_bucket_t;
 
 typedef struct {
@@ -224,8 +229,13 @@ void del_cent(dns_cent_t *cent);
 
 dns_cent_t *copy_cent(dns_cent_t *cent  DBGPARAM);
 
-/* unsigned long get_serial(void); */
+#if 0
+unsigned long get_serial(void);
+#endif
 
+/* have_rr tests whether a cache entry has at least one record of type tp.
+   Only use if T_MIN <= tp <=T_MAX
+*/
 inline static int have_rr(dns_cent_t *cent, int tp)
 {
 	rr_set_t *rrset=cent->rr[tp-T_MIN];
