@@ -1,24 +1,24 @@
 /* consts.c - Common config constants & handling
 
    Copyright (C) 2000, 2001 Thomas Moestl
-   Copyright (C) 2002, 2003, 2005, 2006 Paul A. Rombouts
+   Copyright (C) 2002, 2003, 2005, 2006, 2007 Paul A. Rombouts
 
-This file is part of the pdnsd package.
+  This file is part of the pdnsd package.
 
-pdnsd is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+  pdnsd is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 3 of the License, or
+  (at your option) any later version.
 
-pdnsd is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+  pdnsd is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with pdsnd; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+  You should have received a copy of the GNU General Public License
+  along with pdnsd; see the file COPYING. If not, see
+  <http://www.gnu.org/licenses/>.
+*/
 
 #include <config.h>
 #include <stdlib.h>
@@ -38,10 +38,12 @@ static const namevalue_t const_dic[]={
 	{"domain",      C_DOMAIN},
 	{"excluded",    C_EXCLUDED},
 	{"exec",        C_EXEC},
+	{"fail",        C_FAIL},
 	{"false",       C_OFF},
 	{"fqdn_only",   C_FQDN_ONLY},
 	{"if",          C_IF},
 	{"included",    C_INCLUDED},
+	{"negate",      C_NEGATE},
 	{"no",          C_OFF},
 	{"none",        C_NONE},
 	{"off",         C_OFF},
@@ -82,7 +84,9 @@ static const char *const_names[]={
 	"simple_only",
 	"fqdn_only",
 	"auth",
-	"domain"
+	"domain",
+	"fail",
+	"negate"
 };
 
 /* compare two strings.
