@@ -3,22 +3,22 @@
    Copyright (C) 2000 Thomas Moestl
    Copyright (C) 2003, 2005 Paul A. Rombouts
 
-This file is part of the pdnsd package.
+  This file is part of the pdnsd package.
 
-pdnsd is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+  pdnsd is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 3 of the License, or
+  (at your option) any later version.
 
-pdnsd is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+  pdnsd is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with pdsnd; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+  You should have received a copy of the GNU General Public License
+  along with pdnsd; see the file COPYING. If not, see
+  <http://www.gnu.org/licenses/>.
+*/
 
 /* $Id: hash.h,v 1.3 2001/05/09 17:51:52 tmm Exp $ */
 
@@ -66,7 +66,7 @@ inline static void mk_dns_hash()
 }
 
 dns_cent_t *dns_lookup(const unsigned char *key, dns_hash_loc_t *loc);
-void add_dns_hash(dns_cent_t *data, dns_hash_loc_t *loc);
+int add_dns_hash(dns_cent_t *data, dns_hash_loc_t *loc);
 dns_cent_t *del_dns_hash_ent(dns_hash_loc_t *loc);
 dns_cent_t *del_dns_hash(const unsigned char *key);
 void free_dns_hash_bucket(int i);
