@@ -1,24 +1,24 @@
 /* main.c - Command line parsing, intialisation and server start
 
    Copyright (C) 2000, 2001 Thomas Moestl
-   Copyright (C) 2002, 2003, 2004, 2005, 2006 Paul A. Rombouts
+   Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Paul A. Rombouts
 
-This file is part of the pdnsd package.
+  This file is part of the pdnsd package.
 
-pdnsd is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+  pdnsd is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 3 of the License, or
+  (at your option) any later version.
 
-pdnsd is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+  pdnsd is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with pdsnd; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+  You should have received a copy of the GNU General Public License
+  along with pdnsd; see the file COPYING. If not, see
+  <http://www.gnu.org/licenses/>.
+*/
 
 /* in order to use O_NOFOLLOW on Linux: */
 /* #define _GNU_SOURCE */
@@ -77,18 +77,19 @@ char *conf_file=CONFDIR"/pdnsd.conf";
 static const char info_message[] =
 	
 	"pdnsd - dns proxy daemon, version " VERSION "\n\n"
+	"Copyright (C) 2000, 2001 Thomas Moestl\n"
+	"Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Paul A. Rombouts\n\n"
 	"pdnsd is free software; you can redistribute it and/or modify\n"
 	"it under the terms of the GNU General Public License as published by\n"
-	"the Free Software Foundation; either version 2, or (at your option)\n"
-	"any later version.\n\n"
+	"the Free Software Foundation; either version 3 of the License, or\n"
+	"(at your option) any later version.\n\n"
 	"pdnsd is distributed in the hope that it will be useful,\n"
 	"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
 	"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
 	"GNU General Public License for more details.\n\n"
 	"You should have received a copy of the GNU General Public License\n"
-	"along with pdsnd; see the file COPYING.  If not, write to\n"
-	"the Free Software Foundation, 59 Temple Place - Suite 330,\n"
-	"Boston, MA 02111-1307, USA.\n";
+	"along with pdsnd; see the file COPYING.  If not, see\n"
+	"<http://www.gnu.org/licenses/>.\n";
 
 
 /* the help page */
@@ -109,7 +110,7 @@ static const char help_message[] =
 	"-h\t\t--or--\n"
 	"--help\t\tprint this help page and exit.\n"
 	"-V\t\t--or--\n"
-	"--version\tprint version information and exit.\n"
+	"--version\tprint version and license information and exit.\n"
 	"--pdnsd-user\tprint the user pdnsd will run as and exit.\n"
 	"-s\t\t--or--\n"
 	"--status\tEnable status control socket in the cache directory.\n"
